#ifndef OSM_CONF_CORE_H
#define OSM_CONF_CORE_H

#include <librnd/core/conf.h>
#include <librnd/core/color.h>

typedef struct {
	const struct { /* appearance */
		struct { /* color */
			RND_CFT_COLOR layer[16];       /* default layer colors; when a new layer is created, a color from this list is assigned initially */
			RND_CFT_COLOR attached;
			RND_CFT_COLOR selected;
		} color;
		RND_CFT_BOOLEAN bbox_debug;      /* enable rendering bounding box of objects, for debugging */
	} appearance;

	const struct { /* editor */
		RND_CFT_INTEGER click_time;            /* default time for click expiration, in ms */
		RND_CFT_BOOLEAN thin_draw;             /* if set, ways on the screen are drawn as center-lines and polygons are not filled. This is useful for editing. */
		RND_CFT_BOOLEAN offline;               /* do not attempt to make network connections */
		RND_CFT_REAL merge_way_endp_dist2;     /* when two ways are to be merged and their endpoints are not the same but are very close, merge them; this is the max distance^2 in meter for the merge to happen, so a value of 4 means if the endpoints are closer than 2 meter they are merged */
		RND_CFT_INTEGER buffer_number;         /* index of the current buffer */
		struct {
			RND_CFT_BOOLEAN disable_negative;    /* selection box behaviour: disable the negative-direction selection - any selection box will select only what's fully within the box */
			RND_CFT_BOOLEAN symmetric_negative;  /* selection box behaviour: when set, the selection direction is considered negative only if the box has negative size in the X direction */
		} selection;
	} editor;

	const struct { /* tagging */
		RND_CFT_LIST node_visual_types;        /* ordered list of tag keys, preferred ones first, to determine the visual type of a node object */
		RND_CFT_LIST way_visual_types;         /* ordered list of tag keys, preferred ones first, to determine the visual type of a way object */
	} tagging;

	const struct { /* rc */
		RND_CFT_BOOLEAN error_on_bad_cli_files;
		const struct home { /* rc */
			RND_CFT_REAL lat;  /* initial position if nothing is loaded: latitude */
			RND_CFT_REAL lon;  /* initial position if nothing is loaded: longitude */
		} home;
	} rc;
} conf_core_t;


void conf_core_init();
extern conf_core_t conf_core;

#endif
