/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer
 *  Copyright (C) 2024 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

#ifndef OSM_HISTORY_H
#define OSM_HISTORY_H

#include <genlist/gendlist.h>
#include "osm_typedefs.h"
#include "htIp.h"

struct osm_history_s {
	htIp_t id2obj[OSM_OBJ_vmax]; /* value is always a vtp0_t with multiple versions of the same object */
};

typedef struct osm_timeline_s {
	const osm_any_obj_t *subject; /* one of the objects this item is added for (each changeset is added only once) */
	osm_changeset_t *changeset;   /* changeset causing this change (including date of change) */
	vtp0_t chobjs;                /* objects changed in this changeset */
	gdl_elem_t link;
} osm_timeline_t;

void osm_history_init(osm_history_t *hst);
void osm_history_uninit(osm_history_t *hst);

void osm_history_add(osm_history_t *hst, osm_any_obj_t *obj);

/* Collect and insert timeline events in the ordered list in dst */
void osm_history_insert_timeline_for_obj(osm_history_t *hst, gdl_list_t *dst, osm_any_obj_t *obj);

/* Free a whole timeline, leaving tl uninitialized */
void osm_timeline_free(gdl_list_t *tl);

/* Copy objects from history onto a layer for visualisation */
void osm_history_draw_reset(osm_design_t *osm);
void osm_history_draw_obj(osm_design_t *osm, osm_history_t *hst, osm_any_obj_t *obj);
void osm_history_draw_objs(osm_design_t *osm, osm_history_t *hst, vtp0_t *objs);

/* Centralized storage for now */
extern osm_history_t osm_hist;
extern int osm_hist_inited;

#endif

