/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer
 *  Copyright (C) 2023,2024 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

#include <librnd/config.h>
#include <librnd/hid/hid.h>
#include <librnd/hid/hid_inlines.h>

#include "conf_core.h"
#include "obj_changeset.h"
#include "obj_common.h"
#include "projection.h"
#include "draw.h"

static void osm_changeset_free_fields(osm_any_obj_t *obj)
{
	osm_tags_uninit(&obj->changeset.curr->tags);
	if (obj->changeset.curr != &obj->changeset.orig) {
		free(obj->changeset.curr);
		obj->changeset.curr = &obj->changeset.orig;
	}
}

static void osm_changeset_draw(osm_any_obj_t *obj, rnd_hid_gc_t gc, const rnd_hid_expose_ctx_t *region, rnd_xform_t *xform, int overlay)
{
	rnd_coord_t x1 = obj->changeset.min_x, y1 = obj->changeset.min_y;
	rnd_coord_t x2 = obj->changeset.max_x, y2 = obj->changeset.max_y;

	rnd_hid_set_line_cap(gc, rnd_cap_round);
	rnd_hid_set_line_width(gc, -3);
	rnd_render->set_color(gc, rnd_color_magenta);

	rnd_render->draw_line(gc, x1, y1, x2, y1);
	rnd_render->draw_line(gc, x2, y1, x2, y2);
	rnd_render->draw_line(gc, x2, y2, x1, y2);
	rnd_render->draw_line(gc, x1, y2, x1, y1);
}

static void osm_changeset_draw_attached(osm_any_obj_t *obj, rnd_hid_gc_t gc)
{
}

static void osm_changeset_bbox(osm_any_obj_t *obj)
{
	lldeg2rnd(obj->changeset.curr->min_lon, obj->changeset.curr->min_lat, &obj->changeset.min_x, &obj->changeset.min_y);
	lldeg2rnd(obj->changeset.curr->max_lon, obj->changeset.curr->max_lat, &obj->changeset.max_x, &obj->changeset.max_y);


	obj->changeset.hdr.bbox.x1 = obj->changeset.min_x;
	obj->changeset.hdr.bbox.x2 = obj->changeset.min_y;
	obj->changeset.hdr.bbox.y1 = obj->changeset.max_x;
	obj->changeset.hdr.bbox.y2 = obj->changeset.max_y;
	obj->changeset.hdr.bbox_valid = 1;
}

static osm_any_obj_t *osm_changeset_alloc(void) { return (osm_any_obj_t *)osm_changeset_new(); }

static int osm_changeset_isc_box(const osm_any_obj_t *o, const osm_rtree_box_t *box)
{
	return 0;
}

static osm_tags_t *osm_changeset_get_tags(osm_any_obj_t *o)
{
	return &o->changeset.curr->tags;
}

void osm_changeset_data_copy(osm_changeset_data_t *dst, const osm_changeset_data_t *src, int deep)
{
	dst->min_lat = src->min_lat;
	dst->min_lon = src->min_lon;
	dst->max_lat = src->max_lat;
	dst->max_lon = src->max_lon;
	if (deep)
		osm_tags_dup(&dst->tags, &src->tags);
}

#if 0
static void osm_changeset_fork(osm_changeset_t *nd)
{
	assert(nd->curr == &nd->orig);
	nd->curr = malloc(sizeof(osm_changeset_data_t));
	osm_changeset_data_copy(nd->curr, &nd->orig, 1);
}

static void osm_changeset_unfork(osm_changeset_t *nd)
{
	assert(nd->curr != &nd->orig);
	osm_tags_uninit(&nd->curr->tags);
	free(nd->curr);
	nd->curr = &nd->orig;
}
#endif

void osm_changeset_pre_edit(osm_any_obj_t *o, int geo, int auto_fork)
{
}


void osm_changeset_post_edit(osm_any_obj_t *o, int tags_changed)
{
}

void osm_changeset_referee_changesets(osm_any_obj_t *o, vtp0_t *dst, int update_att)
{
}

static long osm_changeset_referee_idx(osm_any_obj_t *parent, osm_any_obj_t *child)
{
	return -1;
}


static void osm_changeset_diffresult(osm_any_obj_t *o, osm_id_t new_id, osm_id_t new_version)
{
}

static void osm_changeset_diffresult_referees(osm_any_obj_t *o, htdr_t *dr, int permit_del, int undoable)
{
}

static void osm_changeset_pre_remove_referee(osm_any_obj_t *o, osm_objtype_t rem_ty, osm_id_t rem_id, int undoable)
{
}

static void osm_changeset_post_remove(osm_any_obj_t *o, int undoable)
{
}

osm_any_obj_t *osm_changeset_dup_into(osm_any_obj_t *o, osm_layer_t *dst, osm_id_t new_id)
{
	return NULL; /* not a layer object */
}

static const osm_objcalls_t osm_changeset_calls = {
	osm_changeset_alloc,
	osm_changeset_free_fields,
	osm_changeset_draw,
	osm_changeset_draw_attached,
	osm_changeset_bbox,
	osm_changeset_isc_box,
	osm_changeset_get_tags,
	osm_changeset_pre_edit,
	osm_changeset_post_edit,
	osm_changeset_referee_changesets,
	osm_changeset_referee_idx,
	osm_changeset_diffresult,
	osm_changeset_diffresult_referees,
	osm_changeset_pre_remove_referee,
	osm_changeset_post_remove,
	osm_changeset_dup_into
};

void osm_changeset_init(osm_changeset_t *changeset)
{
	memset(changeset, 0, sizeof(osm_changeset_t));
	osm_objcom_init(&changeset->hdr, OSM_OBJ_CHANGESET, &osm_changeset_calls);
	changeset->curr = &changeset->orig;
	osm_tags_init(&changeset->orig.tags);
}

osm_changeset_t *osm_changeset_new(void)
{
	osm_changeset_t *res = malloc(sizeof(osm_changeset_t));
	osm_changeset_init(res);
	return res;
}
