/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer
 *  Copyright (C) 2023 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

#ifndef OSM_RND_PLUG_IO
#define OSM_RND_PLUG_IO

#include <stdio.h>
#include "data.h"

typedef enum osm_io_type_s { /* bitfield so multiple detections (e.g. design-or-layer) can be handled */
	OSM_IOTYP_DESIGN  = 1,
	OSM_IOTYP_LAYER   = 2
} osm_io_type_t;


typedef struct osm_io_s {
	const char *name;
	int prio; /* the higher the better */
	int (*test_load)(osm_design_t *osm, const char *fn, FILE *f); /* returns non-zero if the file looks good for the plugin */
	int (*load)(osm_design_t *osm, const char *fn, FILE *f);

	int (*save_prio)(const char *fn, const char *fmt, osm_io_type_t type);
	int (*save_design)(osm_design_t *osm, const char *fn);
	int (*save_layer)(osm_design_t *osm, osm_layer_t *ly, const char *fn);
	const char *ext_save_design;
	const char *ext_save_layer;

} osm_io_t;

void osm_io_reg(osm_io_t *io);
void osm_io_unreg(osm_io_t *io);
int osm_io_load(osm_design_t *osm, const char *fn);

int osm_save_design(osm_design_t *osm, const char *fn, const char *fmt);
int osm_save_layer(osm_design_t *osm, osm_layer_t *ly, const char *fn_, const char *fmt);

#endif
