/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer
 *  Copyright (C) 2023 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

/* Manage rendering styles */

#ifndef OSM_STYLE_H
#define OSM_STYLE_H

#include "config.h"
#include <genvector/vtp0.h>
#include <genht/htsp.h>
#include "osm_typedefs.h"
#include "data.h"

#include <librnd/core/global_typedefs.h>

typedef struct osm_style_data_s {
	struct {
		rnd_color_t color;
		rnd_coord_t width;
	} stroke;
	struct {
		rnd_color_t color;
	} fill;
	struct {
		rnd_color_t color;
		rnd_coord_t height;
	} label;
} osm_style_data_t;

struct osm_style_s {
	char *name;     /* for the user */
	long id;        /* for easier referencing; index into osm_styles */
	char zord;      /* target z-order level, see in osm_hdr_s */

	/* how to render in high and low LoD */
	osm_style_data_t high, low;

	/* level of details (3-stage) limits */
	int lod_invis_above;     /* do not render the object if displaying more than this meters-per-pixel (zoomed out too much) */
	int lod_high_below;      /* normally render with low style data; below this meters-per-pixel switch over to high style */

	/* TODO: figure how to do these */
	int base_render_algo;        /* on a given z-order level, all objects are rendered with the base algo first... */
	int overlay_render_algo;     /* ... then with the overlay algo */

	unsigned area:1;
	unsigned fallback:1;         /* set for the default fallback styles */
	
};

/* Recompute the ->style field of an object; typically after tags have changed */
void osm_style_update_obj(osm_any_obj_t *o);

extern vtp0_t osm_styles;
extern htsp_t osm_name2style;

/* fallback: when no other style is found for an object (relations are not rendered) */
extern osm_style_t *osm_def_node_style, *osm_def_way_style;

long osm_style_new(const char *name);

#endif
