/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer - standard tools
 *  Copyright (C) 2024 Tibor 'Igor2' Palinkas
 *  (icons copied from pcb-rnd by the author)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

#include "config.h"

#include <librnd/core/actions.h>
#include <librnd/hid/tool.h>

#include "conf_core.h"
#include "crosshair.h"
#include "buffer.h"


static void buffer_init(void)
{
	TODO("check if buffer_number is in range and fix it if not");
}

static void buffer_uninit(void)
{
}

static void buffer_notify_mode(rnd_design_t *hl)
{
	osm_design_t *osm = (osm_design_t *)hl;
	osm_layer_t *ly = osm_layer_get_curr(osm);

	osm_buffer_paste(ly, OSM_PASTEBUFFER, osm->crosshair_x, osm->crosshair_y);
}

static void buffer_draw_attached(rnd_design_t *hl)
{
	osm_design_t *osm = (osm_design_t *)hl;
	osm_layer_t *ly = osm_layer_get_curr(osm);

	osm_buffer_xor_draw(ly, OSM_PASTEBUFFER, osm->crosshair_x, osm->crosshair_y, osm_crosshair_gc);
}

/* XPM */
static const char *buf_icon[] = {
/* columns rows colors chars-per-pixel */
"21 21 3 1",
"  c #000000",
". c #7A8584",
"o c None",
/* pixels */
"oooooooo  oo  ooooooo",
"oooooo.. o  o ..ooooo",
"oooooooo oooo ooooooo",
"oooooo.. oooo ..ooooo",
"oooooooo oooo ooooooo",
"oooooo.. oooo ..ooooo",
"oooooooo oooo ooooooo",
"oooooo.. oooo ..ooooo",
"oooooooo oooo ooooooo",
"oooooo.. oooo ..ooooo",
"oooooooo      ooooooo",
"ooooooooooooooooooooo",
"oo     oo ooo o     o",
"ooo ooo o ooo o ooooo",
"ooo ooo o ooo o ooooo",
"ooo ooo o ooo o    oo",
"ooo    oo ooo o ooooo",
"ooo ooo o ooo o ooooo",
"ooo ooo o ooo o ooooo",
"oo     ooo   oo ooooo",
"ooooooooooooooooooooo"
};

rnd_tool_t osm_rnd_tool_buffer = {
	"buffer", NULL, NULL, 100, buf_icon, RND_TOOL_CURSOR_NAMED("hand"), 0,
	buffer_init,
	buffer_uninit,
	buffer_notify_mode,
	NULL, /* release */
	NULL, /* adjust */
	buffer_draw_attached,
	NULL,
	NULL,
	NULL, /* escape */
	
	0
};
